import { d as MessagesMap, c as MessageContent, e as MessageValues, a as MessageID, b as MessageDescriptor } from './messages-d5f471aa.js';
import { L as LocaleDescriptor, a as Locale } from './locales-22b40efa.js';
import { P as PreferredLocalesSource } from './sources-02c94682.js';
import { E as EventTarget, L as LocaleChangeEvent, a as LocaleLoadEvent, b as ErrorEvent, A as AutomaticStateChangeEvent, c as AfterLocaleChangeEvent, d as Event, e as EventListenerOptions } from './ErrorEvent-881212aa.js';
import { Ref, ComputedRef } from 'vue';
import { IntlShape } from '@formatjs/intl';
import { Options } from 'intl-messageformat';
import { T as TranslateFunction } from './translateFunction-6f001285.js';
import { AsyncComputedRef } from '@braw/async-computed';

type ReverseMap<Map extends Record<keyof Map, PropertyKey>> = {
    [Value in Map[keyof Map]]: {
        [Key in keyof Map]: Map[Key] extends Value ? Key : never;
    }[keyof Map];
};

interface AutomationPartial {
    /**
     * BCP 47 language tag of the locale preferred by one of the automatic
     * sources.
     */
    get preferredLocale(): string;
    /**
     * Adds a preferred locale source.
     *
     * If source is already added, it is removed and added at the new position,
     * that depends on the {@link prepend} argument.
     *
     * @param source Automatic locale source to add.
     * @param prepend Whether to prepend locale source before all other sources.
     *   Default is `false`.
     */
    addSource(source: PreferredLocalesSource, prepend?: boolean): void;
    /**
     * Removes a preferred locale source.
     *
     * @param source Locale source to remove.
     */
    removeSource(source: PreferredLocalesSource): void;
}

interface LocalesPartial {
    /** A reference to a map containing locale data. */
    get $locales(): Ref<Map<LocaleDescriptor, Locale>>;
    /** Read-only reference to the current loading promise. */
    get $loading(): AsyncComputedRef<void>;
    /**
     * Whether the automation is enabled and locale is picked automatically based
     * on one of the preferred locale sources.
     */
    get automatic(): boolean;
    /** @returns Whether the current locale is loaded. */
    get ready(): boolean;
    /** BCP 47 language tag of the currently used locale. */
    get locale(): string;
    /**
     * Finds a descriptor for one of the existing locales. Descriptor objects may
     * contain useful information regarding the presentation of locale.
     *
     * @param localeTag BCP 47 language tag of the locale.
     * @returns Descriptor for the locale or `undefined`, if none exists.
     */
    getLocaleDescriptor(localeTag: string): LocaleDescriptor | undefined;
    /**
     * Adds the provided descriptor to the configuration.
     *
     * If locale for the provided language tag or descriptor already exists, that
     * locale will be replaced, causing all associated with it data to be
     * deleted.
     *
     * @param descriptor Descriptor or a BCP 47 language tag for which the
     *   descriptor is created.
     * @param force Whether to override any existing locale.
     * @returns Descriptor that was added.
     * @throws Error if locale with the same locale tag exists but `force` is not
     *   provided or is set to `false`.
     */
    addLocale(descriptor: string | LocaleDescriptor, force?: boolean): LocaleDescriptor;
    /**
     * Removes the locale from the configuration.
     *
     * @param descriptor Descriptor or BCP 47 language tag of any existing locale.
     * @returns Removed locale descriptor or `null` if no locale was removed.
     */
    removeLocale(descriptor: string | LocaleDescriptor): LocaleDescriptor | null;
    /**
     * Adds messages for the locale, overwriting any existing ones, to the
     * internal messages map.
     *
     * @param descriptor Descriptor or a BCP 47 locale for which the messages are
     *   defined.
     * @param messages A map of messages for the locale for adding. This is
     *   partial, therefore any message keys that were assigned to `undefined`
     *   value will be removed.
     * @throws Error if locale with such tag or descriptor does not exist.
     */
    addMessages(descriptor: string | LocaleDescriptor, messages: Partial<MessagesMap>): void;
    /**
     * Changes the current locale to one of defined locales based on its language
     * tag or descriptor. Alternatively, if `'auto'` is passed as a language tag,
     * switches to one of the preferred locales and toggles automatic mode on.
     *
     * @param descriptor BCP 47 language tag or descriptor of the locale to use.
     * @returns Promise that will resolve when the locale is loaded, or rejected
     *   if locale change is cancelled or any of the locale load event handlers
     *   rejects.
     */
    changeLocale(descriptor: 'auto' | (string & Record<never, never>) | LocaleDescriptor): Promise<void>;
    /**
     * Waits until the locale loading is complete and controller is ready for use.
     *
     * @returns A promise that will be resolved after the default and current
     *   locales finish loading, otherwise it will be rejected with the reason of
     *   unsuccessful load.
     */
    waitUntilReady(): Promise<void>;
}

interface LocaleDataPartial {
    /** Read-only reference to default messages map. */
    get $defaultMessages(): ComputedRef<MessagesMap>;
    /** Read-only reference to messages map for the current locale. */
    get $messages(): ComputedRef<Partial<MessagesMap>>;
    /** Read-only reference to the default resources map. */
    get $defaultResources(): ComputedRef<VueIntlController.LocaleResources>;
    /** Read-only reference to the active */
    get $resources(): ComputedRef<Partial<VueIntlController.LocaleResources>>;
    /** Read-only map of default messages. */
    get defaultMessages(): MessagesMap;
    /** Read-only map of messages for the current locale. */
    get messages(): Partial<MessagesMap>;
    /** Read-only resource map for the default locale. */
    get defaultResources(): VueIntlController.LocaleResources;
    get resources(): Partial<VueIntlController.LocaleResources>;
}

interface IntlController<ControllerType> extends AutomationPartial, ConfigPartial<ControllerType>, LocaleDataPartial, EventTargetPartial<ControllerType>, IntlPartial<ControllerType>, LocalesPartial {
}

type ControllerEvents = LocaleChangeEvent | LocaleLoadEvent | ErrorEvent | AutomaticStateChangeEvent | AfterLocaleChangeEvent;
type EventTargetPartial<T> = EventTarget<ControllerEvents, IntlController<T>>;

type ListenerItemRegular<ControllerType, EventType> = Extract<ControllerEvents, {
    type: EventType;
}> extends never ? never : (this: IntlController<ControllerType>, event: Extract<ControllerEvents, {
    type: EventType;
}>) => any;
type ListenerItemWithOptions<ControllerType, EventType> = {
    listener: ListenerItemRegular<ControllerType, EventType>;
    options?: EventListenerOptions;
};
type ListenerItem<ControllerType, EventType> = ListenerItemWithOptions<ControllerType, EventType> | ListenerItemRegular<ControllerType, EventType>;
type ControllerEventsMap<ControllerType> = {
    [EventType in ControllerEvents extends Event ? ControllerEvents['type'] : never]?: ControllerEvents extends Event ? EventType extends ControllerEvents['type'] ? ListenerItem<ControllerType, EventType> | ListenerItem<ControllerType, EventType>[] : never : never;
};

/** Represents the controller configuration (state). */
interface ControllerConfiguration<ControllerType> {
    /**
     * BCP 47 language tag of the currently used locale.
     *
     * @default 'en-US'
     */
    locale: string;
    /**
     * BCP 47 language tag of the default locale.
     *
     * @default 'en-US'
     */
    defaultLocale: string;
    /**
     * All the locale descriptors.
     *
     * Defaults to newly meta-less generated descriptors for the default and
     * current locales.
     *
     * @default [{ tag: 'en-US' }]
     */
    locales: LocaleDescriptor[];
    /**
     * Whether the automatic language selection is enabled.
     *
     * @default false // Uses configured locale.
     */
    usePreferredLocale: boolean;
    /**
     * An array of preferred locale sources.
     *
     * The order in which sources defines the preference for one particular
     * source.
     *
     * @default [ ] // Fall backs to default locale.
     */
    preferredLocaleSources: PreferredLocalesSource[];
    /** Defines a map of event listeners to auto-bind. */
    listen: ControllerEventsMap<ControllerType>;
    /**
     * Order in which default message is applied during the descriptor
     * normalization.
     *
     * Setting this to `['locale']` will only use the default message from the
     * loaded default locale and never the descriptor.
     *
     * Setting this to `['locale', 'descriptor']` will try to use the default
     * message from the loaded default locale, and if it does not exist, then the
     * default message from the message descriptor will be used.
     *
     * Settings this to `[]` will not use any of the default messages, so if
     * translation for that message is missing, it's ID will be returned.
     *
     * @default ['descriptor'] // Uses default message from descriptor as fallback.
     */
    defaultMessageOrder: ('descriptor' | 'locale')[];
}
interface ConfigPartial<ControllerType> {
    /** Reactive configuration of the controller. */
    get $config(): ControllerConfiguration<ControllerType>;
    /** A read-only array of all available locale descriptors. */
    get availableLocales(): readonly LocaleDescriptor[];
    /** BCP 47 language tag of the default locale. */
    get defaultLocale(): string;
    /** {@link Intl.Locale} for the current locale. */
    get intlLocale(): Intl.Locale;
}

declare const formatAliases: {
    readonly formatDate: "date";
    readonly formatDateTimeRange: "dateTimeRange";
    readonly formatDateToParts: "dateToParts";
    readonly formatDisplayName: "displayName";
    readonly formatList: "list";
    readonly formatListToParts: "listToParts";
    readonly formatMessage: "message";
    readonly formatNumber: "number";
    readonly formatNumberToParts: "numberToParts";
    readonly formatPlural: "plural";
    readonly formatRelativeTime: "relativeTime";
    readonly formatTime: "time";
    readonly formatTimeToParts: "timeToParts";
};
type InvertedMapping = ReverseMap<typeof formatAliases>;
/**
 * Represents a map of formatting functions taken from {@link IntlShape} under a
 * shorter name. This is used to define `$fmt` global variable in Vue components
 * when the plugin is installed, as well as a return value for the
 * `useFormatters` runtime function.
 */
type FormatAliases<T> = {
    [K in keyof InvertedMapping]: IntlShape<T>[InvertedMapping[K]];
};
interface IntlPartial<T> {
    /** Reactive reference to an active {@link IntlShape} instance. */
    get $intl(): ComputedRef<IntlShape<T>>;
    /**
     * Read-only map of formatting functions taken from {@link IntlShape}.
     *
     * @see {@link FormatAliases} for details.
     */
    get formats(): FormatAliases<T>;
    /** Active {@link IntlShape} instance. */
    get intl(): IntlShape<T>;
    /**
     * Improved variant of {@link IntlShape.formatMessage} function that uses
     * {@link LocaleDataPartial.defaultMessages} as fallback if string is missing
     * from the messages of the current locale.
     *
     * @see {@link TranslateFunction} For information about parameters and return values.
     */
    get formatMessage(): TranslateFunction;
    /**
     * Formats a custom message using the provided values.
     *
     * @param message ICU MessageFormat message or its AST.
     * @param values Values to format {@link message} with.
     * @param opts Optional parser options.
     * @returns Normalized formatted message.
     */
    get formatCustomMessage(): (this: void, message: MessageContent, values?: MessageValues, opts?: Options) => string;
    /**
     * Given message descriptor or its ID, normalizes the descriptor according to
     * configuration.
     *
     * @param inputDescriptor Descriptor to normalize.
     * @returns Input descriptor if it conforms to configuration, otherwise a
     *   newly created descriptor.
     */
    normalizeMessageDescriptor<ID extends MessageID>(inputDescriptor: MessageDescriptor<ID> | ID): MessageDescriptor<ID>;
}

export { ControllerConfiguration as C, FormatAliases as F, IntlController as I };
