const AsyncEventSymbol = Symbol("isAsync");
function markAsAsync(event) {
  Object.defineProperty(event, AsyncEventSymbol, {
    configurable: true,
    value: true
  });
}
function isAsyncEvent(event) {
  return AsyncEventSymbol in event && AsyncEventSymbol in event;
}

const CollectableSymbol = Symbol("isCollectable");
const CollectedPropertyKey = "collected";
const CollectPropertyKey = "collect";
function implementCollection(event, collector) {
  let collected = false;
  function wrappedCollector() {
    if (collected) {
      throw new Error("Cannot call collector for a second time");
    }
    try {
      return collector();
    } finally {
      collected = true;
    }
  }
  Object.defineProperties(event, {
    [CollectableSymbol]: {
      configurable: true,
      value: true
    },
    [CollectedPropertyKey]: {
      configurable: true,
      get() {
        return collected;
      }
    },
    [CollectPropertyKey]: {
      configurable: true,
      value: wrappedCollector
    }
  });
}

const Canceled = Symbol("isCancellable");
const CanceledPropertyKey = "canceled";
const CancelPropertyKey = "cancel";
function implementCancelation(event) {
  let cancelled = false;
  Object.defineProperties(event, {
    [Canceled]: {
      configurable: true,
      get() {
        return cancelled;
      }
    },
    [CanceledPropertyKey]: {
      configurable: true,
      get() {
        return cancelled;
      }
    },
    [CancelPropertyKey]: {
      configurable: true,
      value: function cancel() {
        cancelled = true;
      }
    }
  });
}
function isCancelable(event) {
  return Canceled in event;
}
function isCanceled(event) {
  return isCancelable(event) && event[Canceled];
}

class LocaleChangeEvent {
  /**
   * @param previousLocale Previously used locale.
   * @param locale Locale that locale will be changed to.
   * @param automatic Whether the locale change was invoked by automation.
   */
  constructor(previousLocale, locale, automatic) {
    this.previousLocale = previousLocale;
    this.locale = locale;
    this.automatic = automatic;
    this.type = "localechange";
    implementCancelation(this);
  }
}

function createHashMap() {
  return /* @__PURE__ */ Object.create(null);
}

class LocaleLoadEvent {
  /**
   * @param localeDescriptor Locale descriptor that describes active locale.
   * @param locale Locale object which the initial values will be mapped from.
   */
  constructor(localeDescriptor, locale) {
    this.type = "localeload";
    this.locale = localeDescriptor;
    this.messages = locale.messages ?? createHashMap();
    this.resources = locale.resources ?? createHashMap();
    implementCancelation(this);
    implementCollection(this, () => {
      this.cancel();
      return {
        messages: this.messages,
        resources: this.resources
      };
    });
    markAsAsync(this);
  }
  /**
   * Adds provided messages to the the intermediate messages map, overriding any
   * existing values.
   *
   * For better type interference this method uses partial object notation
   * rather than parameters.
   *
   * @param messages Messages to add to the intermediate messages map.
   */
  addMessages(messages) {
    Object.assign(this.messages, messages);
  }
  /**
   * Adds provided resources to the the intermediate resources map, overriding
   * any existing values.
   *
   * For better type interference this method uses partial object notation
   * rather than parameters.
   *
   * @param resources Resources to add to the intermediate resources map.
   */
  addResources(resources) {
    Object.assign(this.resources, resources);
  }
}

class AutomaticStateChangeEvent {
  constructor(state) {
    this.state = state;
    this.type = "automatic";
    implementCancelation(this);
  }
}

class AfterLocaleChangeEvent {
  constructor(previousLocale, locale, automatic) {
    this.previousLocale = previousLocale;
    this.locale = locale;
    this.automatic = automatic;
    this.type = "afterlocalechange";
  }
}

class ErrorEvent {
  // eslint-disable-next-line n/handle-callback-err
  constructor(error, cause, event, listener) {
    this.error = error;
    this.cause = cause;
    this.event = event;
    this.listener = listener;
    this.type = "error";
  }
}

export { AutomaticStateChangeEvent as A, ErrorEvent as E, LocaleChangeEvent as L, implementCollection as a, implementCancelation as b, isCancelable as c, isCanceled as d, LocaleLoadEvent as e, AfterLocaleChangeEvent as f, createHashMap as g, isAsyncEvent as i, markAsAsync as m };
