import { match } from '@formatjs/intl-localematcher';
import { isRef, watch, shallowRef, computed, toRaw, reactive, shallowReadonly, ref } from 'vue';
import { g as createHashMap, i as isAsyncEvent, E as ErrorEvent, d as isCanceled, f as AfterLocaleChangeEvent, e as LocaleLoadEvent, L as LocaleChangeEvent, A as AutomaticStateChangeEvent } from './vue-intl-controller.041c4d65.mjs';
import { createIntlCache, createIntl } from '@formatjs/intl';
import { asyncComputed } from '@braw/async-computed';

function newObject() {
  return /* @__PURE__ */ Object.create(null);
}
function defineGetters(map) {
  const result = newObject();
  for (const property of [
    ...Object.getOwnPropertyNames(map),
    ...Object.getOwnPropertySymbols(map)
  ]) {
    Object.defineProperty(result, property, {
      configurable: true,
      get() {
        return map[property];
      }
    });
  }
  return result;
}
function defineRefGetters(referencesMap) {
  const result = createHashMap();
  for (const propertyKey of [
    ...Object.getOwnPropertyNames(referencesMap),
    ...Object.getOwnPropertySymbols(referencesMap)
  ]) {
    let normalizedKey = propertyKey;
    if (typeof normalizedKey === "string" && normalizedKey.startsWith("$")) {
      normalizedKey = normalizedKey.slice(1);
    }
    Object.defineProperty(result, normalizedKey, {
      configurable: true,
      get: isRef(referencesMap[propertyKey]) ? function referenceGetter() {
        return referencesMap[propertyKey].value;
      } : function getter() {
        return referencesMap[propertyKey];
      }
    });
  }
  return result;
}
function mergeDescriptors(_source1, _source2, _source3, _source4, _source5, _source6, _source7, _source8, _source9) {
  const result = newObject();
  for (const object of arguments) {
    if (object == null)
      continue;
    for (const property of [
      ...Object.getOwnPropertyNames(object),
      ...Object.getOwnPropertySymbols(object)
    ]) {
      Object.defineProperty(
        result,
        property,
        Object.getOwnPropertyDescriptor(object, property)
      );
    }
  }
  return result;
}

function observe(source, callback, options) {
  return watch(source, callback, {
    ...options,
    immediate: true,
    flush: "sync"
  });
}

function usePrefersPartial($config) {
  const $sources = shallowRef([]);
  observe(
    () => $config.preferredLocaleSources,
    (updatedSources) => {
      const entries = $sources.value;
      const newEntries = [];
      if (updatedSources != null) {
        for (const entry of entries) {
          if (!updatedSources.includes(entry.source)) {
            entry.source.uninstall?.();
            entry.installed = false;
          }
        }
        for (const source of updatedSources) {
          const entry = entries.find((it) => it.source === source);
          if (entry == null || !entry.installed) {
            source.install?.();
          }
          newEntries.push({
            installed: true,
            source
          });
        }
      }
      $sources.value = newEntries;
    }
  );
  const $preferredLocale = computed(() => {
    for (const entry of $sources.value) {
      const { source } = entry;
      let value;
      if (isRef(source.prefers)) {
        value = source.prefers.value;
      } else {
        value = source.prefers;
      }
      if (value == null) {
        continue;
      }
      return match(
        value,
        $config.locales.map((it) => it.tag),
        $config.defaultLocale
      );
    }
    return $config.defaultLocale;
  });
  function addSource(source, prepend = false) {
    const filteredSources = [];
    if (prepend)
      filteredSources.push(source);
    if ($config.preferredLocaleSources != null) {
      for (const existingSource of $config.preferredLocaleSources) {
        if (existingSource === source)
          continue;
        filteredSources.push(existingSource);
      }
    }
    if (!prepend)
      filteredSources.push(source);
    $config.preferredLocaleSources = filteredSources;
  }
  function removeSource(source) {
    const filteredSources = [];
    let shallUpdate = false;
    for (const existingSource of $config.preferredLocaleSources) {
      if (toRaw(existingSource) === source) {
        shallUpdate = true;
      } else {
        filteredSources.push(existingSource);
      }
    }
    if (shallUpdate) {
      $config.preferredLocaleSources = filteredSources;
    }
  }
  const refs = { $preferredLocale };
  return mergeDescriptors(defineRefGetters(refs), {
    addSource,
    removeSource
  });
}

function createConfig(initialConfiguration) {
  const defaultLocale = initialConfiguration?.defaultLocale ?? "en-US";
  const locale = initialConfiguration?.locale ?? defaultLocale;
  let locales = initialConfiguration?.locales;
  if (locales == null) {
    locales = [];
    locales.push({ tag: defaultLocale });
    if (locale !== defaultLocale) {
      locales.push({ tag: locale });
    }
  }
  return {
    defaultLocale,
    locale,
    locales,
    usePreferredLocale: initialConfiguration?.usePreferredLocale ?? false,
    preferredLocaleSources: initialConfiguration?.preferredLocaleSources ?? [],
    listen: initialConfiguration?.listen ?? {},
    defaultMessageOrder: initialConfiguration?.defaultMessageOrder ?? [
      "descriptor"
    ]
  };
}
function useConfigPartial(initialConfiguration) {
  const $config = reactive(createConfig(initialConfiguration));
  const $intlLocale = computed(() => new Intl.Locale($config.locale));
  return mergeDescriptors(
    defineGetters({ $config }),
    {
      get availableLocales() {
        return shallowReadonly($config.locales);
      },
      get defaultLocale() {
        return $config.defaultLocale;
      }
    },
    defineRefGetters({ $intlLocale })
  );
}

function useLocaleDataPartial($config, locales) {
  function getLocale(localeTag) {
    const descriptor = locales.getLocaleDescriptor(localeTag);
    if (descriptor == null) {
      throw new Error(`No descriptor exists for locale with tag "${localeTag}"`);
    }
    return locales.$locales.value.get(descriptor);
  }
  const $defaultMessages = computed(
    () => getLocale($config.defaultLocale)?.messages ?? createHashMap()
  );
  const $messages = computed(
    () => getLocale($config.locale)?.messages ?? createHashMap()
  );
  const $defaultResources = computed(
    () => getLocale($config.defaultLocale)?.resources ?? createHashMap()
  );
  const $resources = computed(
    () => getLocale($config.locale)?.resources ?? createHashMap()
  );
  const refs = {
    $defaultMessages,
    $messages,
    $defaultResources,
    $resources
  };
  return mergeDescriptors(defineGetters(refs), defineRefGetters(refs));
}

function useEventTargetPartial(controllerBox) {
  const eventRegister = /* @__PURE__ */ new Map();
  function addEventListener(type, listener, options) {
    if (!eventRegister.has(type)) {
      eventRegister.set(type, /* @__PURE__ */ new Set());
    }
    const entries = eventRegister.get(type);
    for (const entry of entries) {
      if (entry.listener === listener) {
        entries.delete(entry);
      }
    }
    entries.add({
      listener,
      options: {
        ...options
      }
    });
  }
  function removeEventListener(type, listener) {
    const entries = eventRegister.get(type);
    if (entries == null)
      return;
    for (const value of entries) {
      if (value.listener === listener) {
        entries.delete(value);
      }
    }
  }
  function dispatchEvent(event) {
    const eventType = event.type;
    const entries = eventRegister.get(eventType);
    let canceled = false;
    const allowAsync = isAsyncEvent(event);
    async function impl() {
      if (entries == null)
        return;
      const sortedListeners = [...entries.values()].sort(
        (a, b) => (b.options.priority ?? 0) - (a.options.priority ?? 0)
      );
      for (const entry of sortedListeners) {
        if (canceled && !(entry.options.always ?? false)) {
          continue;
        }
        try {
          const ret = entry.listener.call(controllerBox.value, event);
          if (allowAsync)
            await ret;
        } catch (err) {
          if (eventType === "error" || (eventRegister.get("error")?.size ?? 0) < 1) {
            console.error("Uncaught", err);
          } else {
            const wrappedError = new Error(
              `An error occurred while calling the event listener for "${eventType}"`,
              { cause: err }
            );
            dispatchEvent(
              new ErrorEvent(wrappedError, err, event, entry.listener)
            );
          }
        }
        if (entry.options.once ?? false) {
          entries.delete(entry);
        }
        if (!canceled)
          canceled = isCanceled(event);
      }
    }
    const completion = impl();
    if (entries != null && entries.size === 0) {
      eventRegister.delete(event.type);
    }
    if (allowAsync) {
      return completion.then(
        () => !canceled,
        () => false
      );
    } else {
      return !canceled;
    }
  }
  return {
    addEventListener,
    removeEventListener,
    dispatchEvent
  };
}

const formatAliases = {
  formatDate: "date",
  formatDateTimeRange: "dateTimeRange",
  formatDateToParts: "dateToParts",
  formatDisplayName: "displayName",
  formatList: "list",
  formatListToParts: "listToParts",
  formatMessage: "message",
  formatNumber: "number",
  formatNumberToParts: "numberToParts",
  formatPlural: "plural",
  formatRelativeTime: "relativeTime",
  formatTime: "time",
  formatTimeToParts: "timeToParts"
};
function useIntlPartial($config, localeDataPartial) {
  const $formats = ref(createHashMap());
  const intlCache = createIntlCache();
  const $intl = computed(
    () => createIntl(
      {
        locale: $config.locale,
        defaultLocale: $config.defaultLocale,
        messages: localeDataPartial.messages
      },
      intlCache
    )
  );
  observe($intl, (intl) => {
    const formats = $formats.value;
    for (const [intlProp, alias] of Object.entries(formatAliases)) {
      formats[alias] = intl[intlProp];
    }
  });
  function normalizeOutput(output) {
    if (typeof output === "string")
      return output;
    if (Array.isArray(output)) {
      let normalized = "";
      for (const item of output) {
        normalized += String(item);
      }
      return normalized;
    }
    return String(output);
  }
  function normalizeMessageDescriptor(inputDescriptor) {
    let descriptor;
    if (typeof inputDescriptor === "string") {
      descriptor = { id: inputDescriptor };
      if ($config.defaultMessageOrder.includes("locale")) {
        descriptor.defaultMessage = localeDataPartial.defaultMessages[inputDescriptor];
      }
      return descriptor;
    }
    for (const source of $config.defaultMessageOrder) {
      if (source === "descriptor") {
        if (inputDescriptor.defaultMessage == null)
          continue;
        return inputDescriptor;
      }
      if (source === "locale") {
        const defaultMessage = localeDataPartial.defaultMessages[inputDescriptor.id];
        if (defaultMessage == null)
          continue;
        descriptor = {
          ...inputDescriptor,
          defaultMessage
        };
        return descriptor;
      }
    }
    if (inputDescriptor.defaultMessage == null) {
      return inputDescriptor;
    }
    return descriptor = {
      ...inputDescriptor,
      defaultMessage: void 0
    };
  }
  const formatMessage = function formatMessage2(descriptor, values, opts) {
    let result = "";
    result = $intl.value.formatMessage(
      normalizeMessageDescriptor(descriptor),
      values,
      opts
    );
    return normalizeOutput(result);
  };
  function formatCustomMessage(message, values, opts) {
    const intl = $intl.value;
    return normalizeOutput(
      intl.formatters.getMessageFormat(message, intl.locale, intl.formats, opts).format(values)
    );
  }
  return mergeDescriptors(
    defineRefGetters({ $formats, $intl }),
    defineGetters({
      $intl,
      formatMessage,
      formatCustomMessage,
      normalizeMessageDescriptor
    })
  );
}

function includes(iterator, search) {
  for (const item of iterator) {
    if (item === search)
      return true;
  }
  return false;
}
function find(iterator, predicate) {
  for (const item of iterator) {
    if (predicate(item))
      return item;
  }
  return void 0;
}

/**
 * Performs equality by iterating through keys on an object.
 *
 * @license MIT Copyright (c) Meta Platforms, Inc. and affiliates.
 *   https://github.com/facebook/react/blob/ff9f943741671b6d83d732b2131d3f7e7d3c54c8/LICENSE
 * @returns `false` when any key has values which are not strictly equal between
 *   the arguments, or `true` when the values of all keys are strictly equal.
 */
function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }
  if (typeof objA !== "object" || objA === null || typeof objB !== "object" || objB === null) {
    return false;
  }
  const keysA = Object.keys(objA);
  const keysB = Object.keys(objB);
  if (keysA.length !== keysB.length) {
    return false;
  }
  for (let i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }
  return true;
}

function useLocalesPartial(initialLocaleData, $config, eventTarget, prefersPartial) {
  const $locales = shallowRef(
    new Map(initialLocaleData)
  );
  observe(
    () => $config.locales,
    (newLocales) => {
      const knownLocales = new Map($locales.value);
      const tags = /* @__PURE__ */ new Set();
      for (const locale of newLocales) {
        if (tags.has(locale.tag)) {
          throw new Error(
            `Locale descriptor with tag "${locale.tag}" has already been added`
          );
        }
        tags.add(locale.tag);
        if (includes(knownLocales.keys(), locale))
          continue;
        knownLocales.set(locale, {
          messages: createHashMap(),
          resources: createHashMap()
        });
      }
      for (const knownLocale of knownLocales.keys()) {
        if (!newLocales.includes(knownLocale)) {
          knownLocales.delete(knownLocale);
        }
      }
      $locales.value = knownLocales;
    }
  );
  function getLocaleDescriptor(localeTag) {
    return find(
      $locales.value.keys(),
      (descriptor) => descriptor.tag === localeTag
    );
  }
  function getLocaleDescriptorAssertive(localeTag) {
    const descriptor = getLocaleDescriptor(localeTag);
    if (descriptor == null) {
      throw new Error(
        `Cannot find the locale descriptor for the locale "${localeTag}"`
      );
    }
    return descriptor;
  }
  function getLocale(descriptor) {
    return $locales.value.get(descriptor);
  }
  function getLocaleAssertive(descriptor) {
    const locale = getLocale(descriptor);
    if (locale == null) {
      throw new Error(
        `Locale for the provided descriptor of "${descriptor.tag}" does not exist`
      );
    }
    return locale;
  }
  function getLocaleByTagAssertive(localeTag) {
    const descriptor = getLocaleDescriptorAssertive(localeTag);
    const locale = getLocaleAssertive(descriptor);
    return [locale, descriptor];
  }
  const $automatic = computed(() => $config.usePreferredLocale);
  const $locale = computed(() => $config.locale);
  function computeInitialChange() {
    const automatic = $config.usePreferredLocale;
    const localeTag = automatic ? prefersPartial.preferredLocale : $config.locale;
    return [...getLocaleByTagAssertive(localeTag), automatic];
  }
  const $pendingLocaleChange = shallowRef(computeInitialChange());
  function loadLocale(locale, descriptor) {
    const event = new LocaleLoadEvent(descriptor, locale);
    async function callEventAndGetResult() {
      if (!await eventTarget.dispatchEvent(event)) {
        throw new Error(
          `Cannot load locale data for the locale "${descriptor.tag}": load event is cancelled`
        );
      }
      return event.collect();
    }
    return [callEventAndGetResult(), event.cancel.bind(null)];
  }
  const $defaultLocale = computed(
    () => getLocaleByTagAssertive($config.defaultLocale)
  );
  let lastDefaultLocale = null;
  const $defaultLocaleLoading = asyncComputed(
    async function loadDefaultLocale() {
      const defaultLocale = $defaultLocale.value;
      if (shallowEqual(defaultLocale, lastDefaultLocale))
        return;
      const [locale, descriptor] = $defaultLocale.value;
      const [localeData, cancelLoading] = loadLocale(locale, descriptor);
      this.onCancel(cancelLoading);
      Object.assign(locale, await localeData);
      lastDefaultLocale = defaultLocale;
    }
  );
  observe($locales, () => {
    $pendingLocaleChange.value = computeInitialChange();
  });
  let lastLocaleChange = null;
  const $loading = asyncComputed({
    watch: () => $pendingLocaleChange.value,
    async get(pendingLocaleChange) {
      if (shallowEqual(pendingLocaleChange, lastLocaleChange))
        return;
      const [locale, descriptor, automated] = pendingLocaleChange;
      const pendingLocale = [locale, descriptor];
      if (shallowEqual(pendingLocale, $defaultLocale.value)) {
        await $defaultLocaleLoading.promise;
      } else {
        const [lastLocale, lastDescriptor] = lastLocaleChange ?? [];
        if (!shallowEqual(pendingLocale, [lastLocale, lastDescriptor])) {
          const [localeData, cancelLoading] = loadLocale(locale, descriptor);
          this.onCancel(cancelLoading);
          Object.assign(locale, await localeData);
        }
      }
      $config.locale = descriptor.tag;
      eventTarget.dispatchEvent(
        new AfterLocaleChangeEvent(
          lastLocaleChange?.[1] ?? null,
          descriptor,
          automated
        )
      );
      lastLocaleChange = pendingLocaleChange;
    }
  });
  function canChangeLocale(descriptor, automatically) {
    const previousDescriptor = lastLocaleChange?.[1] ?? null;
    return eventTarget.dispatchEvent(
      new LocaleChangeEvent(previousDescriptor, descriptor, automatically)
    );
  }
  observe(
    () => ({
      isEnabled: $automatic.value,
      localeTag: prefersPartial.preferredLocale
    }),
    ({ isEnabled, localeTag }) => {
      if (!isEnabled)
        return;
      const [locale, descriptor] = getLocaleByTagAssertive(localeTag);
      if (!canChangeLocale(descriptor, true))
        return;
      $pendingLocaleChange.value = [locale, descriptor, true];
    }
  );
  function addLocale(descriptor, force) {
    const normalizedDescriptor = typeof descriptor === "string" ? { tag: descriptor } : descriptor;
    const newLocales = [];
    for (const locale of $config.locales) {
      if (locale.tag === normalizedDescriptor.tag) {
        if (!force) {
          throw new Error(`Locale "${normalizedDescriptor.tag}" already exists`);
        }
      } else {
        newLocales.push(locale);
      }
    }
    newLocales.push(normalizedDescriptor);
    $config.locales = newLocales;
    return normalizedDescriptor;
  }
  function removeLocale(descriptor) {
    const index = $config.locales.findIndex(
      typeof descriptor === "string" ? (locale) => locale.tag === descriptor : (locale) => toRaw(locale) === descriptor
    );
    return index === -1 ? null : $config.locales.splice(index, 1)[0] ?? null;
  }
  function addMessages(descriptor, messages) {
    let locale;
    if (typeof descriptor === "string") {
      locale = getLocaleByTagAssertive(descriptor)[0];
    } else {
      locale = getLocaleAssertive(descriptor);
    }
    if (locale.messages == null) {
      locale.messages = createHashMap();
    }
    Object.assign(locale.messages, messages);
  }
  function canToggleAutomation(state) {
    return eventTarget.dispatchEvent(new AutomaticStateChangeEvent(state));
  }
  async function waitUntilReady() {
    await $defaultLocaleLoading.promise;
    await $loading.promise;
  }
  async function changeLocale(descriptor) {
    let newLocale;
    if (descriptor === "auto") {
      if (!canToggleAutomation(true)) {
        throw new Error("Enabling of automatic mode has been cancelled");
      }
    } else {
      if (typeof descriptor === "string") {
        newLocale = getLocaleByTagAssertive(descriptor);
      } else {
        newLocale = [getLocaleAssertive(descriptor), descriptor];
      }
      if ($automatic.value && !canToggleAutomation(false)) {
        throw new Error("Disabling of automatic mode has been cancelled");
      }
      if (!canChangeLocale(newLocale[1], false)) {
        throw new Error(`Locale change to "${newLocale[1].tag}" was cancelled`);
      }
    }
    $config.usePreferredLocale = newLocale == null;
    if (newLocale != null)
      $pendingLocaleChange.value = [...newLocale, false];
    return waitUntilReady();
  }
  return mergeDescriptors(
    defineGetters({ $loading, $defaultLocaleLoading, $locales }),
    defineRefGetters({ $automatic, $locale }),
    {
      get ready() {
        return $loading.fulfilled;
      },
      getLocaleDescriptor,
      addLocale,
      removeLocale,
      addMessages,
      changeLocale,
      waitUntilReady
    }
  );
}

function keys(value) {
  return Object.keys(value);
}
function isListenerWithOptions(value) {
  return typeof value === "object" && value != null && !Array.isArray(value) && Object.prototype.hasOwnProperty.call(value, "listener");
}
function normalizeListeners(listenersRaw) {
  const listeners = [];
  if (listenersRaw == null)
    return listeners;
  if (isListenerWithOptions(listenersRaw)) {
    listeners.push(listenersRaw);
    return listeners;
  }
  if (Array.isArray(listenersRaw)) {
    for (const rawListener of listenersRaw) {
      if (isListenerWithOptions(rawListener)) {
        listeners.push(rawListener);
      } else {
        listeners.push({
          listener: rawListener
        });
      }
    }
  } else {
    listeners.push({
      listener: listenersRaw
    });
  }
  return listeners;
}
function useDeclarativeEvents($config, eventTarget) {
  const knownListenersByType = /* @__PURE__ */ Object.create(null);
  function getKnownListeners(type) {
    if (knownListenersByType[type] == null) {
      knownListenersByType[type] = [];
    }
    return knownListenersByType[type];
  }
  function evict(type, item) {
    const knownListeners = knownListenersByType[type];
    if (knownListeners == null)
      return;
    const itemIndex = knownListeners.findIndex((it) => it === item);
    if (itemIndex !== -1)
      knownListeners.splice(itemIndex, 1);
    if (knownListeners.length === 0)
      delete knownListenersByType[type];
  }
  observe(
    () => $config.listen,
    (listenersByType) => {
      for (const eventType of keys(listenersByType)) {
        if (listenersByType[eventType] == null)
          continue;
        const knownListeners = getKnownListeners(eventType);
        const listeners = normalizeListeners(
          listenersByType[eventType]
        );
        for (const listenerItem of listeners) {
          const knownListenerItemIndex = knownListeners.findIndex(
            (it) => it.listener === listenerItem.listener
          );
          if (knownListenerItemIndex !== -1) {
            const knownListenerItem = knownListeners[knownListenerItemIndex];
            if (shallowEqual(listenerItem.options, knownListenerItem.options)) {
              continue;
            }
            eventTarget.removeEventListener(eventType, listenerItem.listener);
            knownListeners.splice(knownListenerItemIndex, 1);
          }
          let wrappedListener;
          if (listenerItem.options?.once ?? false) {
            const originalListener = listenerItem.listener;
            wrappedListener = function(...args) {
              try {
                return originalListener.call(this, ...args);
              } finally {
                evict(eventType, listenerItem);
              }
            };
          } else {
            wrappedListener = listenerItem.listener;
          }
          eventTarget.addEventListener(
            eventType,
            wrappedListener,
            listenerItem.options
          );
          knownListeners.push(listenerItem);
        }
        for (let i = 0; i < knownListeners.length; i++) {
          const knownListenerItem = knownListeners[i];
          const activeListenerItemIndex = listeners.findIndex(
            (it) => it.listener === knownListenerItem.listener && shallowEqual(it.options, knownListenerItem.options)
          );
          if (activeListenerItemIndex === -1) {
            eventTarget.removeEventListener(
              eventType,
              knownListenerItem.listener
            );
            knownListeners.splice(i, 1);
            i -= 1;
          }
        }
        if (knownListeners.length === 0)
          delete knownListenersByType[eventType];
      }
    }
  );
}

function createController(initialConfiguration, initialLocaleData) {
  const controllerBox = {
    value: null
  };
  const configPartial = useConfigPartial(initialConfiguration);
  const eventTargetPartial = useEventTargetPartial(controllerBox);
  useDeclarativeEvents(configPartial.$config, eventTargetPartial);
  const prefersPartial = usePrefersPartial(configPartial.$config);
  const localesPartial = useLocalesPartial(
    initialLocaleData,
    configPartial.$config,
    eventTargetPartial,
    prefersPartial
  );
  const dataPartial = useLocaleDataPartial(
    configPartial.$config,
    localesPartial
  );
  const intlPartial = useIntlPartial(
    configPartial.$config,
    dataPartial
  );
  controllerBox.value = mergeDescriptors(
    configPartial,
    localesPartial,
    intlPartial,
    dataPartial,
    prefersPartial,
    eventTargetPartial
  );
  return controllerBox.value;
}

export { createController as c };
